@@ -294,12 +294,12 @@ class GringoClaspBase(object):
         
         self.optimization = optimization
         
-    def __parse_witnesses__(self, parser, witnesses, key):
+    def __parse_witnesses__(self, parser, witnesses):
         accu = []
         for answer in witnesses:
-            ts = parser.parse(" ".join(answer[key]))
-            if answer.has_key('Opt'):
-                ts.score = answer['Opt']
+            ts = parser.parse(" ".join(answer['Value']))
+            if answer.has_key('Costs'):
+                ts.score = answer['Costs']
                 
             accu.append(ts)
             
@@ -307,9 +307,9 @@ class GringoClaspBase(object):
         
     def __get_witnesses_key__(self,result):
         key = 'Value'
-        if result['Stats'].has_key('Brave'):
+        if result['Models'].has_key('Brave'):
             key = 'Brave'
-        elif result['Stats'].has_key('Cautious'):
+        elif result['Models'].has_key('Cautious'):
             key = 'Cautious'
             
         return key
@@ -346,13 +346,13 @@ class GringoClaspBase(object):
                         
     def __solve__(self, nmodels, grounding, opts = [], json=True):
         try:
+	    opts = opts + ['--stats']
             addoptions = []
             if self.clasp_options:
                 addoptions = self.clasp_options.split()
 
             if json:
                 opts = opts + ['--outf=2']
-                
             self._clasp = subprocess.Popen(
                 filter_empty_str([self.clasp_bin] + addoptions + [str(nmodels)] + opts),
                 stdin=subprocess.PIPE,
@@ -379,43 +379,42 @@ class GringoClaspBase(object):
                 
     def run(self, programs, nmodels = 1, collapseTerms=True, collapseAtoms=True, additionalProgramText=None, callback=None):
         grounding = self.__ground__(programs, additionalProgramText)
-        if self.optimization:
-            solving = self.__solve__(0, grounding)
-        else:
-            solving = self.__solve__(nmodels, grounding)
+        #if self.optimization:
+            #solving = self.__solve__(0, grounding)
+        #else:
+            #solving = self.__solve__(nmodels, grounding)
+        solving = self.__solve__(nmodels, grounding)
         
         parser = Parser(collapseTerms,collapseAtoms,callback)
         res = json.loads(solving)
         key = self.__get_witnesses_key__(res)
-        
+
         if res['Result'] == "SATISFIABLE":
             if key == 'Value':
-                witnesses = res['Witnesses']
+                witnesses = res['Call'][0]['Witnesses']
             else:
-                witnesses = [res['Witnesses'][-1]]
+                witnesses = [res['Call'][0]['Witnesses'][-1]]
 
-            accu = self.__parse_witnesses__(parser, witnesses, key)            
+            accu = self.__parse_witnesses__(parser, witnesses)            
                 
         elif res['Result'] == "OPTIMUM FOUND":
-            optimum = res['Witnesses'][-1]
-            if nmodels == 1:
-                witnesses = [optimum]  
-            elif (nmodels != 1) and self.clasp_options.find('--opt-all') == -1:
-                optarg = '--opt-all=' + ','.join(map(str, optimum['Opt']))            
-                solving = self.__solve__(nmodels, grounding, [optarg])
-                optall = json.loads(solving)
-            
-                if key == 'Value':
-                    witnesses = optall['Witnesses']
-                else:
-                    witnesses = [optall['Witnesses'][-1]]
+            if key == 'Value':
+                if nmodels == 1:
+		  witnesses = [res['Call'][0]['Witnesses'][-1]]
+                elif (nmodels != 1):
+		  numopts= res['Models']['Optimal']
+		  if numopts==0 : 
+		    print 'WARNING: OPTIMUM FOUND but zero optimals'
+		    witnesses=[]
+		  else :
+		    offset =len(res['Call'][0]['Witnesses'])-numopts
+		    witnesses = res['Call'][0]['Witnesses'][offset:]
+                
             else:
-                if key == 'Value':
-                    witnesses = res['Witnesses']
-                else:
-                    witnesses = [optimum]
-            
-            accu = self.__parse_witnesses__(parser, witnesses, key)
+                witnesses = [res['Call'][0]['Witnesses'][-1]]
+                
+           
+            accu = self.__parse_witnesses__(parser, witnesses)
             
         else:
             accu = []
@@ -425,224 +424,64 @@ class GringoClaspBase(object):
 class GringoClasp(GringoClaspBase):
     pass
 
-class GringoClaspOpt(GringoClaspBase):
-    def __init__(self, clasp_bin = root + '/bin/clasp', clasp_options = '',
-                       gringo_bin = root + '/bin/gringo', gringo_options = '', 
-                       optimization = True):
-                       
-        super(GringoClaspOpt, self).__init__(clasp_bin, clasp_options, gringo_bin, gringo_options, optimization)
-    
+        
 class GringoHClasp(GringoClaspBase):
     def __init__(self, clasp_bin = root + '/bin/hclasp', clasp_options = '',
                        gringo_bin = root + '/bin/gringo', gringo_options = '', 
                        optimization = False):
                    
         super(GringoHClasp, self).__init__(clasp_bin, clasp_options, gringo_bin, gringo_options, optimization)
+        
+    def run(self, programs, nmodels = 1, collapseTerms=True, collapseAtoms=True, additionalProgramText=None, callback=None):
+        grounding = self.__ground__(programs, additionalProgramText)
+
+        solving = self.__solve__(nmodels, grounding)
+        
+        parser = Parser(collapseTerms,collapseAtoms,callback)
+        res = json.loads(solving)
+        #key = self.__get_witnesses_key__(res)
+
+        if res['Result'] == "SATISFIABLE":
+            #if key == 'Value':
+            witnesses = res['Witnesses']
+            #else:
+                #witnesses = [res['Call'][0]['Witnesses'][-1]]
+
+            accu = self.__parse_witnesses__(parser, witnesses)            
+                
+        #elif res['Result'] == "OPTIMUM FOUND":
+            #if key == 'Value':
+                #if nmodels == 1:
+		  #witnesses = [res['Call'][0]['Witnesses'][-1]]
+                #elif (nmodels != 1):
+		  #numopts= res['Models']['Optimal']
+		  #if numopts==0 : 
+		    #print 'WARNING: OPTIMUM FOUND but zero optimals'
+		    #witnesses=[]
+		  #else :
+		    #offset =len(res['Call'][0]['Witnesses'])-numopts
+		    #witnesses = res['Call'][0]['Witnesses'][offset:]
+                
+            #else:
+                #witnesses = [res['Call'][0]['Witnesses'][-1]]
+                
+           
+            #accu = self.__parse_witnesses__(parser, witnesses)
             
-    def __parse_witnesses__(self, parser, witnesses, key):
+        else:
+            accu = []
+
+        return accu            
+            
+            
+    def __parse_witnesses__(self, parser, witnesses):
         accu = []
         for answer in witnesses:
-            atoms = filter(lambda atom: not atom.startswith('_'), answer[key])
+            atoms = filter(lambda atom: not atom.startswith('_'), answer['Value'])
             ts = parser.parse(" ".join(atoms))
             if answer.has_key('Opt'):
                 ts.score = answer['Opt']
             accu.append(ts)
         return accu
-                   
         
-class GringoHClaspOpt(GringoHClasp):
-    def __init__(self, clasp_bin = root + '/bin/hclasp', clasp_options = '',
-                       gringo_bin = root + '/bin/gringo', gringo_options = '', 
-                       optimization = True):
-                   
-        super(GringoHClaspOpt, self).__init__(clasp_bin, clasp_options, gringo_bin, gringo_options, optimization)
-
-      
-class GringoClaspD(GringoClasp):
-    def __init__(self, clasp_bin = root + '/bin/claspD', clasp_options = '',
-                       gringo_bin = root + '/bin/gringo', gringo_options = '', 
-                       optimization = False):
-                   
-        super(GringoClaspD, self).__init__(clasp_bin, clasp_options, gringo_bin, gringo_options, optimization)
-        
-    def __get_witnesses_key__(self,result):
-        if result['Models'].has_key('Brave'):
-            key = 'Brave'
-        elif result['Models'].has_key('Cautious'):
-            key = 'Cautious'
-        else:
-            key = 'Value'
-            
-        return key
-
-class GringoUnClasp(GringoClaspBase):
-    def __init__(self, clasp_bin = root + '/bin/unclasp', clasp_options = '',
-                       gringo_bin = root + '/bin/gringo', gringo_options = '', 
-                       optimization = True):
-                   
-        super(GringoUnClasp, self).__init__(clasp_bin, clasp_options, gringo_bin, gringo_options, optimization)
-        
-
-    def run(self, programs, nmodels = 1, collapseTerms=True, collapseAtoms=True, additionalProgramText=None, callback=None):
-        grounding = self.__ground__(programs, additionalProgramText)
-        solving = self.__solve__(nmodels, grounding, json=False)
-
-        lines = solving.split('\n')
-        parser = Parser(collapseTerms,collapseAtoms,callback)
-        accu = []
-        if "OPTIMUM FOUND" in lines:
-	    #print 'sat',lines[-9]
-            optimum = parser.parse(lines[-9])
-            optimum.score = tuple(map(int, lines[-8][13:].split()))
-            accu = [optimum]
-        if "SATISFIABLE" in lines:
-	    #print 'sat',lines[-10]
-            optimum = parser.parse(lines[-10])
-            optimum.score = tuple(map(int, lines[-9][13:].split()))
-            accu = [optimum]
-
-        return accu
-
-class GringoUnClaspOpt(GringoClaspBase):
-    def __init__(self, *args, **keywords):
-        GringoClaspBase.__init__(self, *args, **keywords)
-        self.clasp_bin = root + '/bin/unclasp'
-
-    def run(self, programs, nmodels = 1, collapseTerms=True, collapseAtoms=True, additionalProgramText=None):
-        '''
-        allows for the following modes of optimization:
-        nmodels=0: get all optimal models
-        nmodels=1: get first optimal model (-n 0)
-        nmodels>1: get first K optimal models
-                   (this enumerates all optimal models using --opt-all)
-        (in both cases non-optimal models are discarded)
-
-        returns tuple:
-        (quality-tuple, list of optimal models)
-
-        the default setting returns the first optimal model
-        '''
-        # options need to be filtered to work with Popen
-        # since gringo/clasp doesn't like whitespace before numbers parameter
-        additionalPrograms = []
-        if additionalProgramText != None:
-          (fd, fn) = tempfile.mkstemp('.lp')
-          file = os.fdopen(fd,'w')
-          file.write(str(additionalProgramText))
-          file.close()
-          additionalPrograms.append(fn)
-          
-        self._gringo = subprocess.Popen(
-            filter_empty_str([self.gringo_bin] +
-              self.gringo_options.split() + programs + additionalPrograms),
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
-        # get ground program
-        (groundout,self.gringo_stderr) = self._gringo.communicate()
-        if self._gringo.returncode not in self.gringo_noerror_retval:
-          raise RuntimeError("gringo terminated with %d and error %s" %
-            (self._gringo.returncode, self.gringo_stderr))
-        #
-        # now we first do clasp -n 0 to get weights of the optimal model
-        # (for getting all models, we can later initialize the solver with this
-        # weight and use --opt-all)
-        self._clasp = subprocess.Popen(
-            filter_empty_str([self.clasp_bin,'--quiet=0'] +
-              self.clasp_options.split()),
-            stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
-        # send ground program to clasp and signal end of input
-        self._clasp.stdin.write(groundout)
-        self._clasp.stdin.close()
-        # get resulting answer sets, ignore everything to last (optimal) model
-        lastweightline = None
-        lastmodelline = None
-        l = self._clasp.stdout.readline()
-        while l != '' and l[:13] != 'OPTIMUM FOUND':
-            #debug(l.strip())
-            if l.startswith('Answer'):
-                # get model
-                lastmodelline = self._clasp.stdout.readline()
-                #debug(lastmodelline.strip())
-                # get weights
-                lastweightline = self._clasp.stdout.readline()
-                #debug(lastweightline.strip())
-                if lastweightline[:12] != 'Optimization':
-                  # not an optimization problem!
-                  raise RuntimeError("expected 'Optimization' line, got '%s' " %
-                    lastweightline.strip() + "(not an optimization problem?)")
-            if l.startswith('UNSATISFIABLE'):
-                return None
-            l = self._clasp.stdout.readline()
-
-        # clasp execution error handling
-        self._clasp.wait()
-        self.clasp_stderr = self._clasp.stderr.read()
-        if self._clasp.returncode not in self.clasp_noerror_retval:
-          raise Exception("got error %d from clasp: '%s' gringo: '%s'" % \
-            (self._clasp.returncode, self.clasp_stderr, self.gringo_stderr))
-
-        # received content error handling
-        if lastweightline == None or lastmodelline == None:
-          claspstdout = self._clasp.stdout.read()
-          self.clasp_stderr = self._clasp.stderr.read()
-          raise RuntimeError("did not find optimum! stdout='%s' stderr='%s'" %
-            (claspstdout, self.clasp_stderr))
-        # interpret weight of optimal model
-        optimal_weights = lastweightline[13:].split()
-        optimal_weights = tuple(map(int,optimal_weights))
-        #print >>sys.stderr, "optimal weights %s" % (optimal_weights,)
-        #
-        # we know the optimal weight -> get optimal model(s) next
-        #
-        parser = Parser(collapseTerms=collapseTerms,collapseAtoms=collapseAtoms)
-        # if we only need 1 model, parse and return it
-        if nmodels == 1:
-          #print >>sys.stderr, "parsing %s" % (lastmodelline,)
-          model = parser.parse(lastmodelline)
-          return (optimal_weights, [model])
-        # we need more models -> call solver again with preset opt value
-        # --opt-val=<optimal_weights> and --opt-all and -n <nmodels>
-        # -> we get a maximum of <nmodels> models and they are all optimal
-        optarg = '--opt-all=' + ','.join(map(str,optimal_weights))
-        args = filter_empty_str([self.clasp_bin, '-n', str(nmodels),
-          optarg] + self.clasp_options.split())
-        #print >>sys.stderr, "calling clasp again with %s" % (args,)
-        self._clasp = subprocess.Popen(args, stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        # send ground program to clasp and signal end of input
-        self._clasp.stdin.write(groundout)
-        self._clasp.stdin.close()
-        # parse and return all resulting answer sets
-        accu = []
-        l = self._clasp.stdout.readline()
-        while l != '':
-            #print >>sys.stderr, "DBG got line '%s'" % (l,)
-            if l.startswith('Answer'):
-                # get model
-                modelline = self._clasp.stdout.readline()
-                # get weights
-                weightline = self._clasp.stdout.readline()
-                assert(weightline == lastweightline)
-                #print >>sys.stderr, "parsing modelline %s" % (modelline,)
-                model = parser.parse(modelline)
-                # build result
-                accu.append(model)
-            if l.startswith('Models'):
-                # statistics start here
-                break
-            l = self._clasp.stdout.readline()
-
-        # clasp execution error handling
-        self._clasp.wait()
-        # unfortunately the clasp stats appear on stdout and not on stderr, so we have to pfusch here
-        self.clasp_stderr = self._clasp.stdout.read()
-        self.clasp_stderr += self._clasp.stderr.read()
-        if self._clasp.returncode not in self.clasp_noerror_retval:
-          raise Exception("got error %d from clasp: '%s'" % (self._clasp.returncode, self.clasp_stderr))
-
-        self._clasp = None
-        self._gringo = None
-        return (optimal_weights, accu)
         
\ No newline at end of file
